#pragma once

#ifndef _TMANAGER_
#define _TMANAGER_

#define TMANAGER_ERROR_NO			  0
#define TMANAGER_ERROR_NOTFOUND		  1
#define TMANAGER_ERROR_EXISTS		  2
#define TMANAGER_ERROR_NOFREEINDEX	  3
#define TMANAGER_ERROR_INVALIDINDEX   4




///////////////////////////////////////////////////////////////////////////////
// TManager
///////////////////////////////////////////////////////////////////////////////
template <class T, int n> class TManager
{
public:
	TManager(void);
	~TManager(void);	

	int Add(const T* p_T);
	int Delete(const T* p_T);
	int Delete(int index);
	int DeleteAll();
	
	T* Get(int index);
	T* Get(const T* p_T);
	
	int	GetCount();
	int GetError();
	char *GetErrorString();

protected:
	int	 error;
	T*   p_T[n];	
};




template <class T, int n>TManager<T, n>::TManager(void)
{
	error = TMANAGER_ERROR_NO;	

	for(int i = 0; i < n; i++)
		p_T[i] = 0x00000000;	
}




template <class T, int n>TManager<T, n>::~TManager(void)
{
	// Free all pointers that were still in use
	for(int i = 0; i < n; i++)
	{
		if(p_T[i] != 0x00000000)
		{
			delete p_T[i];
			p_T[i] = 0x00000000;
		}
	}	
}




template <class T, int n> int TManager<T, n>::Add(const T* p_T)
{
	// Check wheter it already exists
	for(int i = 0; i < n; i++)
	{
		if(TManager<T, n>::p_T[i] != 0x00000000)
		{
			if(*TManager<T, n>::p_T[i] == *p_T)
			{
				error = TMANAGER_ERROR_EXISTS;
				return 0;
			}
		}
	}
	
	// The object wasn't a member of the list yet... 
	// so find a free index for it and copy the data to it
	for(i = 0; i < n; i++)
	{
		if(TManager<T, n>::p_T[i] == 0x00000000)
		{
			TManager<T, n>::p_T[i] = new T;
			*TManager<T, n>::p_T[i] = *p_T;
			return 1;
		}
	}
	
	// If we reach this point then no free index was found for the object
	error = TMANAGER_ERROR_NOFREEINDEX;
	return 0;
}




template <class T, int n> int TManager<T, n>::Delete(const T* p_T)
{
	// Try to find it in the list
	for(int i = 0; i < n; i++)
	{
		if(*TManager<T, n>::p_T[i] == *p_T)
		{
			delete TManager<T, n>::p_T[i];
			TManager<T, n>::p_T[i] = 0x00000000;
			return 1;
		}
	}

	// It wasn't a member of the list
	error = TMANAGER_ERROR_NOTFOUND;
	return 0;
}




template <class T, int n> int TManager<T, n>::Delete(int index)
{
	// First make sure that the index contains valid data
	if(TManager<T, n>::p_T[index] != 0x00000000)
	{
		delete TManager<T, n>::p_T[index];
		TManager<T, n>::p_T[index] = 0x00000000;
		return 1;
	}
	else
	{
		// The index pointed to a NULL pointer
		error = TMANAGER_ERROR_INVALIDINDEX;	
		return 0;
	}
		
	error = TMANAGER_ERROR_NOTFOUND;
	return 0;
}




template <class T, int n> int TManager<T, n>::DeleteAll()
{
	for(int i = 0; i < n; i++)
	{
		if(p_T[i] != 0x00000000)
		{
			delete p_T[i];
			p_T[i] = 0x00000000;
		}
	}	
	
	return 1;
}
	



template <class T, int n> T* TManager<T, n>::Get(int index)
{
	// Make sure that the data contains valid information
	if(TManager<T, n>::p_T[index] == 0x00000000)
	{
		error = TMANAGER_ERROR_INVALIDINDEX;	
		return 0x00000000;
	}

	return TManager<T, n>::p_T[index];
}




template <class T, int n> T* TManager<T, n>::Get(const T* p_T)
{
	for(int i = 0; i < n; i++)
	{
		if(TManager<T, n>::p_T[i] != 0x00000000)
		{
			if(*TManager<T, n>::p_T[i] == *p_T)
			{
				return TManager<T, n>::p_T[i];
			}
		}
	}

	error = TMANAGER_ERROR_NOTFOUND;
	return 0;
}




template <class T, int n> int TManager<T, n>::GetCount()
{
	return n;
}




template <class T, int n> int TManager<T, n>::GetError()
{
	return error;
}




template <class T, int n> char *TManager<T, n>::GetErrorString()
{
	switch(error)
	{
	case TMANAGER_ERROR_NO:
		return "TManager -> No error\n";
	case TMANAGER_ERROR_NOTFOUND:
		return "TManager -> Object not found!\n";
	case TMANAGER_ERROR_EXISTS:
		return "TManager -> Object already exists\n";
	case TMANAGER_ERROR_NOFREEINDEX:
		return "TManager -> No free index for object left\n";
	case TMANAGER_ERROR_INVALIDINDEX: 
		return "TManager -> Invalid index (pointer is NULL)\n";
	}

	return "TManager -> Undefined error!\n";
}








///////////////////////////////////////////////////////////////////////////////
// TPersistentManager
///////////////////////////////////////////////////////////////////////////////
template <class T, int n> class TPersistentManager
{
public:
	TPersistentManager(void);
	~TPersistentManager(void);	

	int Add(const T* p_T);
	int Delete(const T* p_T);
	int Delete(int index);
	int DeleteAll();
	
	T* Get(int index);
	T* Get(const T* p_T);
	
	int	GetCount();
	int GetError();
	char *GetErrorString();

private:
	static int refcount;
	static TManager<T, n>	*p_m_tManager;	
};

template <class T, int n> int TPersistentManager<T, n>::refcount = 0;
template <class T, int n> TManager<T, n>* TPersistentManager<T, n>::p_m_tManager = 0x00000000;




template <class T, int n>TPersistentManager<T, n>::TPersistentManager(void)
{
	if(refcount == 0)
	{
		p_m_tManager = new TManager<T, n>;
	}

	refcount++;
}




template <class T, int n>TPersistentManager<T, n>::~TPersistentManager(void)
{
	refcount--;

	if(refcount == 0)
	{
		delete p_m_tManager;
		p_m_tManager = 0x00000000;
	}
}




template <class T, int n> int TPersistentManager<T, n>::Add(const T* p_T)
{
	return p_m_tManager->Add(p_T);
}




template <class T, int n> int TPersistentManager<T, n>::Delete(const T* p_T)
{
	return p_m_tManager->Delete(p_T);
}




template <class T, int n> int TPersistentManager<T, n>::Delete(int index)
{
	return p_m_tManager->Delete(index);
}




template <class T, int n> int TPersistentManager<T, n>::DeleteAll()
{
	return p_m_tManager->DeleteAll();
}
	



template <class T, int n> T* TPersistentManager<T, n>::Get(int index)
{
	return p_m_tManager->Get(index);
}




template <class T, int n> T* TPersistentManager<T, n>::Get(const T* p_T)
{
	return p_m_tManager->Get(p_T);
}




template <class T, int n> int TPersistentManager<T, n>::GetCount()
{
	return p_m_tManager->GetCount();
}




template <class T, int n> int TPersistentManager<T, n>::GetError()
{
	return p_m_tManager->GetError();
}




template <class T, int n> char *TPersistentManager<T, n>::GetErrorString()
{
	return p_m_tManager->GetErrorString();
}

#endif	// _TMANAGER_